﻿B4i=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=8.1
@EndOfDesignText@
Sub Class_Globals
	'Private xlog As XLogger

End Sub

Public Sub Initialize
	'xlog.Initialize("iAccessibility")
End Sub

#if B4i

' Sets the label text of a given Label as AccessibilityLabel for the given View
' pLabel - the label to set for the pView
' pView - the View that needs the label
' uses: AccessibilityLabel - A succinct label in a localized string that identifies the accessibility element.
' see: https://developer.apple.com/documentation/objectivec/nsobject/1615181-accessibilitylabel?language=objc
Public Sub setAccessibilityLabel(pLabel As B4XView, pView As B4XView)
	'xlog.startMethod("setAccessibilityLabel")
	
	If pLabel = Null Or Not(pLabel.IsInitialized) Or pView = Null Or Not(pView.IsInitialized) Then
		''xlog.warn($"Failed due to missing or null param."$).finishMethod
		Log($"Failed due to missing or null param."$)
		Return
	End If
	
	Try
		pView.As(NativeObject).SetField("accessibilityLabel", pLabel.Text)
	Catch
		'xlog.err($"Failed due to exception: ${LastException.Message}"$)
		Log($"Failed due to exception: ${LastException.Message}"$)
	End Try
	
	''xlog.finishMethod
End Sub


' Sets a given Text as AccessibilityLabel for the given View
' pstrText - the label text to set for the pView
' pView - the View that needs the label text
' uses: AccessibilityLabel - A succinct label in a localized string that identifies the accessibility element.
' see: https://developer.apple.com/documentation/objectivec/nsobject/1615181-accessibilitylabel?language=objc
Public Sub setAccessibilityLabelText(pstrText As String, pView As View)
	'xlog.startMethod("setAccessibilityLabelText")
	
	If pView = Null Or Not(pView.IsInitialized) Then
		'xlog.warn($"Failed due to missing or null param."$).finishMethod
		Log($"Failed due to missing or null param."$)
		Return
	End If
	
	Try
		pView.As(NativeObject).SetField("accessibilityLabel", pstrText)
	Catch
		'xlog.err($"Failed due to exception: ${LastException.Message}"$)
		Log($"Failed due to exception: ${LastException.Message}"$)
	End Try
	
	'xlog.finishMethod
End Sub

' Sets the Value of the given View to make it available as AccessibilityValue
' pstrValue - the value to set for the pView
' pView - the View that needs the value
' uses: AccessibilityValue - A localized string that contains the value of the accessibility element.
' see: https://developer.apple.com/documentation/objectivec/nsobject/1615117-accessibilityvalue?language=objc
Public Sub setAccessibilityValue(pstrValue As String, pView As View)
	'xlog.startMethod("setAccessibilityValue")
	
	If pView = Null Or Not(pView.IsInitialized) Then
		'xlog.warn($"Failed due to missing or null param."$).finishMethod
		Log($"Failed due to missing or null param."$)
		Return
	End If
	
	Try
		pView.As(NativeObject).SetField("accessibilityValue", pstrValue)
	Catch
		'xlog.err($"Failed due to exception: ${LastException.Message}"$)
		Log($"Failed due to exception: ${LastException.Message}"$)
	End Try
	
	'xlog.finishMethod
End Sub


' Sets a AccessibilityHint brief description for the given view
' pstrHint - the hint (brief description) to set for the pView
' pView - the View that needs the hint
' uses: AccessibilityHint - A localized string that contains a brief description of the result of performing an action on the accessibility element.
' see: https://developer.apple.com/documentation/objectivec/nsobject/1615093-accessibilityhint?language=objc
Public Sub setAccessibilityHint(pstrHint As String, pView As View)
	'xlog.startMethod("setAccessibilityHint")
	
	If pView = Null Or Not(pView.IsInitialized) Then
		'xlog.warn($"Failed due to missing or null param."$).finishMethod
		Log($"Failed due to missing or null param."$)
		Return
	End If
	
	Try
		pView.As(NativeObject).SetField("accessibilityHint", pstrHint)
	Catch
		'xlog.err($"Failed due to exception: ${LastException.Message}"$)
		Log($"Failed due to exception: ${LastException.Message}"$)
	End Try
	
	'xlog.finishMethod
End Sub

' Set the AccessibilityLanguage for the given view
' pstrLanguageBcp47 - Language code to be set as a AccessibilityLanguage for the given view, use Null to set default language
' pView - View to set the AccessibilityLanguage for
' uses: AccessibilityLanguage - The language To use when speaking the accessibility element's label, value, and hint.
' see: https://developer.apple.com/documentation/objectivec/nsobject/1615192-accessibilitylanguage?language=objc
' see also: https://www.rfc-editor.org/info/bcp47
Public Sub setAccessibilityLanguage(pstrLanguageBcp47 As String, pView As View)
	'xlog.startMethod("setAccessibilityLanguage")
	
	If pView = Null Or Not(pView.IsInitialized) Then
		'xlog.warn($"Failed due to missing or null param."$).finishMethod
		Log($"Failed due to missing or null param."$)
		Return
	End If
	
	Try
		'If XStringUtils.IsNullOrEmpty(pstrLanguageBcp47) Then
		If pstrLanguageBcp47 = "" Or pstrLanguageBcp47 = Null Then
			' Set default language
			pView.As(NativeObject).SetField("accessibilityLanguage", Null)
		Else
			pView.As(NativeObject).SetField("accessibilityLanguage", pstrLanguageBcp47)
		End If
	Catch
		'xlog.err($"Failed due to exception: ${LastException.Message}"$)
		Log($"Failed due to exception: ${LastException.Message}"$)
	End Try
	
	'xlog.finishMethod
End Sub

' Sets the order of two AccessibilityElements
' Prerequisites: Current view and next view have the same parent panel.
' pCurrent - Current view that is followed by a next element 
' pNext - Next view that
' uses AccessibilityElements- An Array of the accessibility elements in the container. 
' see: https://developer.apple.com/documentation/objectivec/nsobject/1615147-accessibilityelements?language=objc
Public Sub setNextFocus(pCurrent As View, pNext As View)
	'xlog.startMethod("setNextFocus")
	
	Dim NativeMe As NativeObject = Me
	
	If pCurrent = Null Or Not(pCurrent.IsInitialized) Or pNext = Null Or Not(pNext.IsInitialized) Then
		'xlog.warn($"Failed due to missing or null param."$).finishMethod
		Log($"Failed due to missing or null param."$)
		Return
	End If
	
	Try
		If pCurrent.Parent <> pNext.Parent Then
			' TODO
			'pCurrent.As(NativeObject).RunMethod("nextResponder:", Array(pNext))
		Else
			NativeMe.RunMethod("updateAccessibilityOrder::", Array As Object(pCurrent.Parent, pNext))
		End If
	Catch
		'xlog.err($"Failed due to exception: ${LastException.Message}"$)
		Log($"Failed due to exception: ${LastException.Message}"$)
	End Try
	
	'xlog.finishMethod
End Sub

' Sets the order of two AccessibilityElements
' Prerequisites: Current view and next view have the same parent panel.
' pCurrent - Current view that is followed by a next element 
' pNext - Next view that
' uses AccessibilityElements- An Array of the accessibility elements in the container. 
' see: https://developer.apple.com/documentation/objectivec/nsobject/1615147-accessibilityelements?language=objc
Public Sub setNextFocusInPanel(pPanel As View, pNext As View)
	'xlog.startMethod("setNextFocus")
	
	Dim NativeMe As NativeObject = Me
	
	If pPanel = Null Or Not(pPanel.IsInitialized) Or pNext = Null Or Not(pNext.IsInitialized) Then
		'xlog.warn($"Failed due to missing or null param."$).finishMethod
		Log($"Failed due to missing or null param."$)
		Return
	End If
	
	Try
		If pPanel <> pNext.Parent Then
			' TODO
			'pCurrent.As(NativeObject).RunMethod("nextResponder:", Array(pNext))
		Else
			NativeMe.RunMethod("updateAccessibilityOrder::", Array As Object(pPanel, pNext))
		End If
	Catch
		'xlog.err($"Failed due to exception: ${LastException.Message}"$)
		Log($"Failed due to exception: ${LastException.Message}"$)
	End Try
	
	'xlog.finishMethod
End Sub

' Sets the focus on the next view according to the order of the AccessibilityElements
' pCurrent - The current view 
' Returns the view that has the next focus or Null if there is no following element to focus 
Public Sub focusNext(pCurrent As View) As View
	'xlog.startMethod("focusNext")
	
	Dim NativeMe As NativeObject = Me
	Dim vwFocused As View
	
	
	Dim NativeMe As NativeObject = Me
	
	If pCurrent = Null Or Not(pCurrent.IsInitialized) Then
		'xlog.warn($"Failed due to missing or null param."$).finishMethod
		Log($"Failed due to missing or null param."$)
		Return Null
	End If
	
	Try
		vwFocused = NativeMe.RunMethod("focusOnNextAccessibilityElementAfterElement:", Array As Object(pCurrent))
	Catch
		'xlog.err($"Failed due to exception: ${LastException.Message}"$).finishMethod
		Log($"Failed due to exception: ${LastException.Message}"$)
		Return Null
	End Try

	'xlog.finishMethod
	Return vwFocused
End Sub

#If ObjC
#import <UIKit/UIKit.h>
#import <AVFoundation/AVFoundation.h>

// Prozedur, um die Fokus-Reihenfolge in einem Container festzulegen oder fortzusetzen
- (void) updateAccessibilityOrder: (UIView *)container :(UIView *)nextElement {
    NSMutableArray *accessibilityOrder = [NSMutableArray arrayWithArray:container.accessibilityElements];
    
    if (!accessibilityOrder) {
        accessibilityOrder = [NSMutableArray new];
    }
    
    if (nextElement) {
        [accessibilityOrder addObject:nextElement];
    }
	
    // Log-Ausgabe der Elemente
    NSLog(@"Aktualisierte Accessibility-Reihenfolge in Container: %@", container);
    for (UIView *element in accessibilityOrder) {
        NSLog(@"Element: %@", element.accessibilityIdentifier);
    }	
    
    container.accessibilityElements = accessibilityOrder;
}

- (UIView *)focusOnNextAccessibilityElementAfterElement:(UIView *)element {
    UIView *parentView = element.superview;
    if (!parentView) {
        NSLog(@"Das übergebene Element hat kein übergeordnetes Element.");
        return nil;
    }
    
    NSArray *accessibilityElements = parentView.accessibilityElements;
    if (!accessibilityElements || accessibilityElements.count == 0) {
        NSLog(@"Das übergeordnete Element hat keine Accessibility-Elemente.");
        return nil;
    }
    
    NSInteger currentIndex = [accessibilityElements indexOfObject:element];
    if (currentIndex == NSNotFound || currentIndex >= accessibilityElements.count - 1) {
        NSLog(@"Das aktuelle Element ist das letzte Element oder nicht in der Accessibility-Liste.");
        return nil;
    }
    
    UIView *nextElement = accessibilityElements[currentIndex + 1];
    UIAccessibilityPostNotification(UIAccessibilityLayoutChangedNotification, nextElement);
	
	return nextElement;
}


#end if
#End if